import threading 
from ctypes import * 
from . import Gain_Param

Custom_data_num = 7 #Amount of custom data 自定义数据的数量
Custom_data = (c_char_p * Custom_data_num)()  # Number of file pointers to create 创建文件指针的数量
Custom_data[0] = b"CPU_USED"
Custom_data[1] = b"CPU_TEMP"
Custom_data[2] = b"SYSTEM_IP"
Custom_data[3] = b"RX_SPEED"
Custom_data[4] = b"TX_SPEED"
Custom_data[5] = b"GET_MEMORY"
Custom_data[6] = b"GET_SYSTEM_DISK_USED"

class Monitor():
    # System_path Get the current user path 获取当前用户路径
    # NETWORK Set up speed test network card 设置测速网卡
    # GET_SYSTEM_DATA Whether to obtain system data 是否获取系统数据
    # IS12HOUR 0: 24-hour format 1: 12-hour format 0：24小时制 1：12小时制
    # LANGUAGE 0: Chinese 1: English 0：中文 1：英文 
    #The language setting needs to be consistent with the one used by the host computer when creating the theme, otherwise an exception will be displayed.
    # 语言设置需要跟上位机制作主题时使用的一致，否则会显示异常
    # WEATHERTEMPUNIT 0: Celsius 1: Fahrenheit 0：摄氏度 1：华氏度
    def __init__(self,SYSTEM_PATH,NETWORK,GET_SYSTEM_DATA,IS12HOUR,LANGUAGE,WEATHERTEMPUNIT):
        self.dll = CDLL(SYSTEM_PATH + '/2.1inch_USB_Monitor_RPi/Demo/lib/monitor.so')  
        init = self.dll.Monitor_init
        init.restype = c_void_p
        init.argtypes = [c_ubyte,c_ubyte,c_ubyte]
        init(IS12HOUR,LANGUAGE,WEATHERTEMPUNIT) 
        
        self.interval = 0.5  #Refresh system data every 0.5 seconds 0.5秒刷新一次系统数据
        self.timer = None

        self.gain = Gain_Param.Gain_Param()   
        
        if GET_SYSTEM_DATA:
            self.start()

        self.network = NETWORK

    def SetRootDir(self,rootDir):
        self.dll.Monitor_SetRootDir.argtypes = [c_char_p]  
        self.dll.Monitor_SetRootDir.restype = c_void_p
        self.dll.Monitor_SetRootDir(rootDir)

    def SendSystemData(self,key,buff,num):
        try:  
            self.dll.Monitor_sendSystemData.argtypes = [POINTER(c_char_p),POINTER(c_char_p),c_int]  
            self.dll.Monitor_sendSystemData.restype = c_void_p
            self.dll.Monitor_sendSystemData(key,buff,num)
        except Exception as e:  
            pass


    def Delete(self):
        self.dll.Monitor_Delete.argtypes = [c_void_p]  
        self.dll.Monitor_Delete.restype = c_void_p
        self.dll.Monitor_Delete(None)

    def Download(self,name,num,time,rotate,backlight):
        self.dll.Monitor_download_theme.argtypes = [POINTER(c_char_p),c_ubyte, c_int,c_ubyte,c_ubyte]  
        self.dll.Monitor_download_theme.restype = c_void_p
        self.dll.Monitor_download_theme(name,num,time,rotate,backlight)

    def start(self):  
        self.timer = threading.Timer(self.interval, self.handle_interrupt)  
        self.timer.start() 

    def handle_interrupt(self): #timer callback 定时器回调
        try:
            buff = (c_char_p * Custom_data_num)() # Create an array of pointers 创建指针数组
            buff[0] = (str(self.gain.GET_CPU_Used()) + "%").encode()#%
            buff[1] = (str(self.gain.GET_Temp()) + "℃").encode() #℃
            buff[2] = self.gain.GET_IP().encode()
            buff[3] = self.gain.RX_speed(self.network)
            buff[4] = self.gain.TX_speed(self.network)
            buff[5] = (str(self.gain.GET_Memory_Percentage())+ "%").encode() #%
            buff[6] = (str(self.gain.GET_System_Disk_Used())+ "%").encode() #%
            # Update custom data 更新自定义数据
            self.SendSystemData(cast(addressof(Custom_data), POINTER(c_char_p)), cast(addressof(buff), POINTER(c_char_p)) , Custom_data_num) 
            self.start()
        except Exception as e:  
            pass

































